package main

import "fmt"
import "time"

// constants are declared as variables, but using the keyword const
const Pi = 3.14

func main() {
	// multiple variables declaration with default init (0, false, "")
	// types follow variable names
	var x, y, z int

	// multiple assignments are possible
	// semi-colons ; are not necessary, use new-line instead
    x = 1
    x, y, z = 2, 3, 4
	println(x,y,z)
	
	// multiple variables declaration with init
	var i, j int = 1, 2
	println(i, j)

	// multiple intializations
	// using := the keyword var can be omitted (:= within function body only)
	// types are inferred
	n, f, s, b, c := 5, 2.5, "Hello", true, 1+2i
	fmt.Printf("%v (%T)\n", n, n)
	fmt.Printf("%v (%T)\n", f, f)
	fmt.Printf("%v (%T)\n", s, s)
	fmt.Printf("%v (%T)\n", b, b)
	fmt.Printf("%v (%T)\n", c, c)

	// Go base types:
	// bool string
	// int  int8  int16  int32  int64
	// uint uint8 uint16 uint32 uint64 uintptr
	// byte (alias for uint8)
	// rune (alias for int32 - "code point" Unicode)
	// float32 float64
	// complex64 complex128

	// cast: T(v)
	n = int(f)
}

// public names (exported) begin with a capital letter
func Square(a int) int {
	return a*a
}

// functions can return tuples
func Swap(a int, b int) (int, int) {
	return b, a
}

// return values can be given a name
func Split(x, y int) (d, r int) {
	d = x / y
	r = x % y
	return // naked return
}

func main1() {
	fmt.Println(Square(3))
	fmt.Println(Swap(4, 2))
	fmt.Println(Split(17, 3))
}

func Max(x, y int) int {
	// if syntax
	// no (), but {} are mandatory, else {} is optional
	if x > y {
		return x
	} else {
		return y
	}
}

func main2() {
	println(Max(5,3))

	// for syntax
	// no (), but {} are mandatory
	sum := 0
	for i := 1; i < 10; i+=2 {
		sum += i
	}
	println("The sum of odd numbers from 1 to 9 is", sum)

	// no while, use for instead
	mul := 2
	for mul < 1000 {
		mul *= mul
	}
	println(mul)

	// for {...} infinite cyle when no condition
}

func main3() {
	t := time.Now()
	fmt.Println("It's",t)

	// switch syntax
	// evaluate cases top-down, first encountered is taken
	// !! unless fallthrough is used the other cases are not considered
	fmt.Println("When's Saturday?")
	today := t.Weekday()
	switch time.Saturday {
	case today + 0:
		println("Today.")
	case today + 1:
		println("Tomorrow.")
	// possibly a default case
	default:
		println("Too far away.")
	}

	// switch can be used without condition
	switch {
	case t.Hour() < 12:
		println("Good morning!")
	case t.Hour() < 17:
		println("Good afternoon.")
	default:
		println("Good evening.")
	}
}






func main4() {
	// Go has pointers, but no pointer arithmetic
	i := 4
	p := &i
	fmt.Printf("p is %v (%T)\n", p, p)
	fmt.Printf("*p is %v (%T)\n", *p, *p)
	*p = 21              // set i through the pointer
	println("i is", i)   // see the new value of i

	// functions are values too
	twice := func(x int) int { return 2 * x }
	fmt.Printf("twice has type %T\n", twice)
	
	apply := func(f func(int) int, x int) int { return f(x) }
	fmt.Printf("apply has type %T\n", apply)
	
	fmt.Println("apply(twice, 3) =",apply(twice,3))
}

