package main

import "fmt"
import "math"

type Vertex struct {
	X int
	Y int
}

func main() {
	// create a Vertex
	v := Vertex{1, 2}
	fmt.Printf("v is %v (%T)\n", v, v)

	// access a field using dot notation
	v.X = v.Y * 2
	fmt.Println("v is", v)

	q := v // copy
	fmt.Printf("q is %v (%T)\n", q, q)
	q.X = 1e5
	fmt.Println("q is", q)
	fmt.Println("v is still", v) // v is unchanged

	p := &v // alias
	fmt.Printf("p is %v (%T)\n", p, p)
	p.X = 1e5 // p.X same as (*p).X
	fmt.Printf("v is %v (%T)\n", v, v)

	// field assignment
	v1 := Vertex{1, 2} // has type Vertex
	v2 := Vertex{Y: 1} // X:0 is implicit
	v3 := Vertex{}     // X:0 and Y:0
	fmt.Println(v1, v2, v3)
}

// A method is a function with a special receiver argument
func (this Vertex) Dist() float64 {
	return math.Sqrt(float64(this.X*this.X + this.Y*this.Y))
}

// A method is a function with a special receiver argument
// (need a pointer to change the state)
// (avoid copying)
// (value methods can be invoked on pointers and values,
// but pointer methods can only be invoked on pointers)
// (When the value is addressable, the language takes care
// of the common case of invoking a pointer method on a value
// by inserting the address operator automatically)
func (this *Vertex) Move(dx, dy int) {
	if this != nil {
		this.X += dx
		this.Y += dy
	}
}

func main1() {
	v := Vertex{3, 4}
	// method invocation
	fmt.Println(v)
	fmt.Println(v.Dist())
	
	v.Move(6, 8)
	fmt.Println(v)
	fmt.Println(v.Dist())
}


// An interface type is defined as a set of method signatures
type Distable interface {
	Dist() float64
}

type Point int

func (p Point) Dist() float64 {
	return float64(p)
}

func main2() {
	var i1, i2 Distable
	//  A variable of interface type can hold any value that implements those methods
	// (don't need to declare that the type implements the interface)
	i1 = Vertex{3, 4}
	i2 = Point(3)
	fmt.Printf("i1.Dist()=%v\n", i1.Dist())
	fmt.Printf("i2.Dist()=%v\n", i2.Dist())

}
