package main

func main() {
	println("Hello")
	println("World")
}

func main1() {
	// launch a goroutine
	go println("Hello")
	println("World")
	// Hey, what happens? Where is Hello?
	// (when main ends all its goroutines are terminated)
}

// let's sync on a channel
func main2() {
	done := make(chan bool)
	// launch a goroutine
	go func() {
		println("Hello")
		done <- true // send value true on channel done
	}()
	println("World")
	// wait on channel done, ignore received value 
	<-done
}


// Hello takes a channel for exchanging booleans
func Hello(done chan bool) {
	println("Hello")
	done <- true // send value true on channel done
}

func main3() {
	// create a channel for sending booleans
	done := make(chan bool)
	go Hello(done) // launch a goroutine
	println("World")
	// wait on channel done, ignore received value 
	<-done // receive a value from channel done
	// this way World may get printed before Hello
}

func main4() {
	done := make(chan bool)
	go Hello(done)
	<-done 
	// this way Hello gets printed before World
	println("World")
}

func main5() {
	c := make(chan int) // create a channel for sending integers
	c <- 245 // send 245 (but sending is blocking!)
	n := <-c // receive from c and store the value in n
	println(n)
}

func main6() {
	c := make(chan int,1) // create a buffered channel for sending integers
	c <- 245 // send 245 (now sending is not blocking!)
	n := <-c // receive from c and store the value in n
	println(n)
}

func main7() {
	c := make(chan int)
	// do the sending in an anonymous goroutine
	go func() {
		c <- 245
	}()
	n := <-c
	println(n)
}

func main8() {
	c := make(chan int)
	go func() {
		c <- 245
	}()
	// avoid n
	println(<-c)
}

