package main

import (
	"math/rand"
	"time"
)

// objective: show use of close, range

// returns a channel out
// where all int between min and max will be sent
func Gen(min, max int) chan int {
	c = make(chan int)
	go Genc(min, max, out)
	return c
}

func Genc(out chan<- int, min, max int) {
	for i := min; i <= max; i++ {
		out <- i
		time.Sleep(time.Duration(rand.Intn(5e2)) * time.Millisecond)
	}
	close(out)
}

// forwards integers from channel in to channel out
// when in is closed, send true on q
func Fwd(in <-chan int, out chan<- int, q chan<- bool) {
	// range gets all messages from in
	// one at the time
	// until in is closed
	for n := range in {
		out <- n
	}
	q <- true
}

// merges messages coming from c1 and c2 to a new channel c
func Merge(c1, c2 <-chan int) (c chan int) {
	c = make(chan int)

	// installs a forwarder from c1 to c
	q1 := make(chan bool)
	go Fwd(c1, c, q1)

	// installs a forwarder from c2 to c
	q2 := make(chan bool)
	go Fwd(c2, c, q2)

	// waits termination of forwarders to close c
	go func() {
		<-q1
		<-q2
		close(c)
	}()
	return
}

// installs two number generators on c1 and c2
// installs a merger from c1 and c2 to c
// receives all messages from c and prints them on screen
func main() {
	c1 := Gen(1, 10)
	c2 := Gen(1000, 1015)
	c := Merge(c1, c2)
	for n := range c {
		println(n)
	}
	println("Bye")
}
